@@ -3,8 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-# pyre-strict
-
 
 from typing import TYPE_CHECKING, TypeVar
 
@@ -14,3 +12,4 @@ if TYPE_CHECKING:
 
 
 CSTNodeT = TypeVar("CSTNodeT", bound="CSTNode")
+CSTNodeT_co = TypeVar("CSTNodeT_co", bound="CSTNode", covariant=True)
